/*------------------------------------------------*/
/* 05kamera/kamera/main.c                         */
#define _CRT_SECURE_NO_WARNINGS
#define GLEW_STATIC
#include <GL/glew.h>
#include <stdio.h>
#include <GLFW/glfw3.h>
#include "linmath-extended.h"
#include "vertex-data.h"
#include "shader.h"
#include "events.h"

void initVariables(Shader *program);
void initBuffers(Shader *program);
void uninitBuffers();
void ticking(Shader * program, GLFWwindow* window);
void draw(Shader *program, GLFWwindow* window, GLuint *VAO);
void animate();

GLuint VAO[2], BO[5];
struct {
	GLuint uPMatrixLoc;
	GLuint uMVMatrixLoc;
} variables;

int main(void)
{
	GLFWwindow* window;
	/* Initialize the library */
	if (!glfwInit()) {
		return -1;
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	/* Create a windowed mode window and its OpenGL context */
	window = glfwCreateWindow(640, 480, "05 - Kamera", NULL, NULL);
	if (!window)
	{
		printf("Nelze inicializovat kontext\n");
		glfwTerminate();
		return -1;
	}
	/* Make the window's context current */
	glfwMakeContextCurrent(window);
	printf("%s\n", glGetString(GL_VERSION));

	/* Load modern OpenGL functions */
	glewExperimental = GL_TRUE;
	glewInit();

	/* events */
	glfwSetKeyCallback(window, key_callback);
	glfwSetMouseButtonCallback(window, mouse_button_callback);

	//glfwSwapInterval(1);

	Shader program = shader("default.vs", "default.fs");
	if (!program.Program) {
		return -1;
	}

	initVariables(&program);
	initBuffers(&program);

	// setup GL state
	glEnable(GL_DEPTH_TEST);

	ticking(&program, window);

	uninitBuffers();

	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}

void initVariables(Shader *program)
{
	variables.uMVMatrixLoc = program->getUniformLocation(program, "uMVMatrix");
	variables.uPMatrixLoc = program->getUniformLocation(program, "uPMatrix");
}

void initBuffers(Shader *program)
{
	glGenVertexArrays(2, VAO);
	glGenBuffers(5, BO);

	glBindVertexArray(VAO[0]);
	{
		glBindBuffer(GL_ARRAY_BUFFER, BO[0]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(0);

		glBindBuffer(GL_ARRAY_BUFFER, BO[1]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(colors), colors, GL_STATIC_DRAW);
		glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, 4 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(1);
	}
	glBindVertexArray(VAO[1]);
	{
		glBindBuffer(GL_ARRAY_BUFFER, BO[2]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(lines), lines, GL_STATIC_DRAW);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(0);

		glBindBuffer(GL_ARRAY_BUFFER, BO[3]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(lineColors), lineColors, GL_STATIC_DRAW);
		glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, 4 * sizeof(GLfloat), (GLvoid*)0);
		glEnableVertexAttribArray(1);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, BO[4]);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(lineIndices), lineIndices, GL_STATIC_DRAW);
	}
	glBindVertexArray(0); // Unbind VAO
}

void uninitBuffers()
{
	glDeleteVertexArrays(2, VAO);
	glDeleteBuffers(4, BO);
}

void ticking(Shader * program, GLFWwindow* window) {
	/* Loop until the user closes the window */
	while (!glfwWindowShouldClose(window))
	{
		animate();
		draw(program, window, VAO);
		glfwSwapBuffers(window);
		glfwPollEvents();
	}
}

void animate() {
}

void draw(Shader *program, GLFWwindow* window, GLuint *VAO)
{
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);

	glClearColor(1.f, 1.f, 1.f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	program->Use(program);

	mat4x4 uMVMatrix, uPMatrix, uViewMatrix;

	vec3 targetPosition = { 0.0, 0.0, -5.0 };
	mat4x4_perspective(uPMatrix, degToRad(45.0), ((GLfloat)width) / height, 1.0, 100.0);
	mat4x4_translate(uMVMatrix, targetPosition[0], targetPosition[1], targetPosition[2]);
	mat4x4_translate(uViewMatrix, cameraPos.x, cameraPos.y, cameraPos.z);
	mat4x4_invert(uViewMatrix, uViewMatrix);
	mat4x4_mul(uMVMatrix, uViewMatrix, uMVMatrix);

	glUniformMatrix4fv(variables.uMVMatrixLoc, 1, GL_FALSE, (const GLfloat *)uMVMatrix);
	glUniformMatrix4fv(variables.uPMatrixLoc, 1, GL_FALSE, (const GLfloat *)uPMatrix);

	// Vykresli osy
	glBindVertexArray(VAO[1]);
	glDrawElements(GL_TRIANGLES, sizeof(lineIndices) / sizeof(lineIndices[0]), GL_UNSIGNED_INT, 0);
	glBindVertexArray(0);

	// Vykresli jehlan 
	glBindVertexArray(VAO[0]);
	glDrawArrays(GL_TRIANGLES, 0, sizeof(vertices) / sizeof(vertices[0]) / 3);
	glBindVertexArray(0);
}

/*------------------------------------------------*/
