#ifndef LINMATH_PRINT_H
#define LINMATH_PRINT_H

#include "linmath.h"
#include <stdio.h>

static GLfloat degToRad(float deg) {
	return (GLfloat) deg/180.0 * 3.14159;
}

typedef vec3 mat3x3[3];

static inline void mat3x3_from_mat4x4(mat3x3 T, mat4x4 M) {
    float * out = (float *) T;
    float * a = (float *) M;
    out[0] = a[0];
    out[1] = a[1];
    out[2] = a[2];
    out[3] = a[4];
    out[4] = a[5];
    out[5] = a[6];
    out[6] = a[8];
    out[7] = a[9];
    out[8] = a[10];
}

static inline void mat3x3_invert(mat3x3 T, mat3x3 M) {
    float * out = (float *) T;
    float * a = (float *) M;

    float a00 = a[0], a01 = a[1], a02 = a[2],
        a10 = a[3], a11 = a[4], a12 = a[5],
        a20 = a[6], a21 = a[7], a22 = a[8],

        b01 = a22 * a11 - a12 * a21,
        b11 = -a22 * a10 + a12 * a20,
        b21 = a21 * a10 - a11 * a20,

	// Calculate the determinant
	det = a00 * b01 + a01 * b11 + a02 * b21;

	if (!det) { 
	        return; 
	}
	det = 1.0 / det;

	out[0] = b01 * det;
	out[1] = (-a22 * a01 + a02 * a21) * det;
	out[2] = (a12 * a01 - a02 * a11) * det;
	out[3] = b11 * det;
	out[4] = (a22 * a00 - a02 * a20) * det;
	out[5] = (-a12 * a00 + a02 * a10) * det;
	out[6] = b21 * det;
	out[7] = (-a21 * a00 + a01 * a20) * det;
	out[8] = (a11 * a00 - a01 * a10) * det;
}

static inline void mat3x3_transpose(mat3x3 T, mat3x3 M) {
    float * out = (float *) T;
    float * a = (float *) M;
    // If we are transposing ourselves we can skip a few steps but have to cache some values
    if (out == a) {
        float a01 = a[1], a02 = a[2], a12 = a[5];
        out[1] = a[3];
        out[2] = a[6];
        out[3] = a01;
        out[5] = a[7];
        out[6] = a02;
        out[7] = a12;
    } else {
        out[0] = a[0];
        out[1] = a[3];
        out[2] = a[6];
        out[3] = a[1];
        out[4] = a[4];
        out[5] = a[7];
        out[6] = a[2];
        out[7] = a[5];
        out[8] = a[8];
    }
}

static inline void mat4x4_print(mat4x4 T)
{
	int i,j;
	printf("-\n");
	for(i = 0; i < 4; i++) {
		for(j = 0; j < 4; j++) {
			printf("%1.2f   ", T[i][j]);
		}
		printf("\n");
	}
}

static inline void mat3x3_print(mat3x3 T)
{
	int i,j;
	printf("-\n");
	for(i = 0; i < 3; i++) {
		for(j = 0; j < 3; j++) {
			printf("%1.2f   ", T[i][j]);
		}
		printf("\n");
	}
}


#endif
