/*------------------------------------------------*/
/* 10assimp/assimp/mesh.h                         */
#ifndef _MESH_H
#define _MESH_H
#include <GL/glew.h>
#include <stdbool.h>
#include "linmath-extended.h"

typedef struct Mesh Mesh;
typedef struct Model Model;

typedef struct Vertex {
	vec3 Position;
	vec3 Normal;
	vec2 TexCoords;
} Vertex;

typedef struct Texture {
	GLuint id;
	enum aiTextureType type;
	char * path;
} Texture;

typedef struct Mesh {
	GLuint VAO, VBO, EBO;
	Vertex *vertices;
	GLuint *indices;
	Texture *textures;
	size_t numVertices;
	size_t numIndices;
	size_t numTextures;
	bool hasTextures;
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
	vec4 emissiveColor;
	float shininess;
	unsigned int ambientTexCount;
	unsigned int diffuseTexCount;
	unsigned int specularTexCount;
	void (*Draw)(Mesh *mesh, Model * model);
} Mesh;

Mesh constructMesh();
void setupMesh(Mesh * mesh);

#endif
/*------------------------------------------------*/
